@@ -1,5 +1,5 @@
 # localslackirc
-# Copyright (C) 2018-2022 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2023 Salvo "LtWorf" Tomaselli
 #
 # localslackirc is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ import msgparsing
 from diff import seddiff
 
 
-VERSION = '1.17'
+VERSION = '1.20'
 
 
 class IrcDisconnectError(Exception): ...
@@ -83,6 +83,7 @@ MPIM_HIDE_DELAY = datetime.timedelta(days=50)
 class ClientSettings(NamedTuple):
     nouserlist: bool
     autojoin: bool
+    no_rejoin_on_mention: bool
     provider: Provider
     ignored_channels: Set[bytes]
     silenced_yellers: Set[bytes]
@@ -95,7 +96,12 @@ class ClientSettings(NamedTuple):
 
         In that case return None. Otherwise an error string.
         '''
-        if not self.downloads_directory.is_dir():
+        if not self.downloads_directory.exists():
+            try:
+                self.downloads_directory.mkdir(parents=True)
+            except Exception:
+                return f'Unable to create {self.downloads_directory}'
+        elif not self.downloads_directory.is_dir():
             return f'{self.downloads_directory} is not a directory'
         return None
 
@@ -123,6 +129,12 @@ class Client:
         self._mentions_regex_cache: dict[str, Optional[re.Pattern]] = {}  # Cache for the regexp to perform mentions. Key is channel id
         self._annoy_users: dict[str, int] = {} # Users to annoy pretending to type when they type
 
+    def get_mention_str(self) -> str:
+        '''
+        Returns the string that is present in messages when the user is getting mentioned
+        '''
+        return f'<@{self.sl_client.login_info.self.id if self.sl_client.login_info else ""}>'
+
     async def _nickhandler(self, cmd: bytes) -> None:
         if b' ' not in cmd:
             self.nick = b'localslackirc'
@@ -198,28 +210,29 @@ class Client:
         await self.s.drain()
 
     async def _joinhandler(self, cmd: bytes) -> None:
-        _, channel_name_b = cmd.split(b' ', 1)
+        _, channel_names_b = cmd.split(b' ', 1)
 
-        if channel_name_b in self.parted_channels:
-            self.parted_channels.remove(channel_name_b)
+        for channel_name_b in channel_names_b.split(b','):
+            if channel_name_b in self.parted_channels:
+                self.parted_channels.remove(channel_name_b)
 
-        channel_name = channel_name_b[1:].decode()
-        try:
-            slchan = await self.sl_client.get_channel_by_name(channel_name)
-        except Exception:
-            await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to find channel: {channel_name}')
-            return
-
-        if not slchan.is_member:
+            channel_name = channel_name_b[1:].decode()
             try:
-                await self.sl_client.join(slchan)
+                slchan = await self.sl_client.get_channel_by_name(channel_name)
             except Exception:
-                await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to join server channel: {channel_name}')
+                await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to find channel: {channel_name}')
+                continue
 
-        try:
-            await self._send_chan_info(channel_name_b, slchan)
-        except Exception:
-            await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to join channel: {channel_name}')
+            if not slchan.is_member:
+                try:
+                    await self.sl_client.join(slchan)
+                except Exception:
+                    await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to join server channel: {channel_name}')
+
+            try:
+                await self._send_chan_info(channel_name_b, slchan)
+            except Exception:
+                await self._sendreply(Replies.ERR_NOSUCHCHANNEL, f'Unable to join channel: {channel_name}')
 
     async def _send_chan_info(self, channel_name: bytes, slchan: slack.Channel|slack.MessageThread):
         if not self.settings.nouserlist:
@@ -363,6 +376,8 @@ class Client:
     async def _parthandler(self, cmd: bytes) -> None:
         name = cmd.split(b' ')[1]
         self.parted_channels.add(name)
+        if name in self.known_threads:
+            del self.known_threads[name]
 
     async def _awayhandler(self, cmd: bytes) -> None:
         is_away = b' ' in cmd
@@ -607,7 +622,11 @@ class Client:
                 if t.kind == msgparsing.Itemkind.MENTION: # User mention
                     r += (await self.sl_client.get_user(t.val)).name
                 elif t.kind == msgparsing.Itemkind.CHANNEL: # Channel mention
-                    r += '#' + (await self.sl_client.get_channel(t.val)).name_normalized
+                    try:
+                        r += '#' + (await self.sl_client.get_channel(t.val)).name_normalized
+                    except KeyError:
+                        # Slack might decide to send id of channels that do not exist, yes…
+                        r += '#ERROR_MISSING_CHANNEL'
                 elif t.kind == msgparsing.Itemkind.YELL: # Channel shouting
                     if (source not in self.settings.silenced_yellers) and (destination not in self.settings.silenced_yellers):
                          yell = ' [%s]:' % self.nick.decode('utf8')
@@ -669,24 +688,42 @@ class Client:
         except Exception as e:
             log('Error: ', str(e))
             return
-        if dest in self.parted_channels:
-            # Ignoring messages, channel was left on IRC
-            return
-
-        if sl_ev.thread_ts:
-            # Threaded message
-            thread = await self.sl_client.get_thread(sl_ev.thread_ts, sl_ev.channel)
-            dest = b'#' + thread.name.encode('utf8')
-
-            # Join thread channel if needed
-            if dest not in self.known_threads or dest in self.parted_channels:
-                if dest in self.parted_channels:
-                    self.parted_channels.remove(dest)
-                await self._send_chan_info(dest, self.known_threads.get(dest, thread))
-                self.known_threads[dest] = self.known_threads.get(dest, thread)
 
         text = sl_ev.text
 
+        mentioned = (not self.settings.no_rejoin_on_mention) and self.get_mention_str() in text
+
+        if sl_ev.thread_ts:
+            # Threaded message, rewriting the dest
+            thread = await self.sl_client.get_thread(sl_ev.thread_ts, sl_ev.channel)
+            original_dest = dest
+            dest = b'#' + thread.name.encode('utf8')
+
+            if dest in self.parted_channels:
+                if mentioned:
+                    self.parted_channels.remove(dest)
+                else:
+                    # This thread is being ignored
+                    return
+
+            # Join thread channel if needed
+            if dest not in self.known_threads:
+                if original_dest in self.parted_channels and not mentioned:
+                    # Ignoring new threads from a parted channel
+                    # but keeping the known ones active
+                    return
+                await self._send_chan_info(dest, self.known_threads.get(dest, thread))
+                self.known_threads[dest] = self.known_threads.get(dest, thread)
+        elif dest in self.parted_channels:
+            if mentioned:
+                # rejoin channel
+                slchan = await self.sl_client.get_channel(sl_ev.channel)
+                await self._send_chan_info(dest, slchan)
+            else:
+                # Ignoring messages, channel was left on IRC
+                # This ignores also threads on those channels
+                return
+
         if sl_ev.files:
             for f in sl_ev.files:
                 text+=f'\n[file upload] {f.name}\n{f.mimetype} {f.size} bytes\n{f.url_private}'
@@ -768,6 +805,9 @@ class Client:
         else:
             cmdid = cmd
 
+        # commands are case insensitive
+        cmdid = cmdid.upper()
+
         handlers = {
             b'NICK': self._nickhandler,
             b'USER': self._userhandler,
@@ -782,12 +822,12 @@ class Client:
             b'TOPIC': self._topichandler,
             b'KICK': self._kickhandler,
             b'INVITE': self._invitehandler,
-            b'sendfile': self._sendfilehandler,
-            b'annoy': self._annoyhandler,
+            b'SENDFILE': self._sendfilehandler,
+            b'ANNOY': self._annoyhandler,
             b'QUIT': self._quithandler,
             #CAP LS
             b'USERHOST': self._userhosthandler,
-            b'whois': self._whoishandler,
+            b'WHOIS': self._whoishandler,
         }
 
         if cmdid in handlers:
@@ -839,6 +879,8 @@ def main() -> None:
     parser.add_argument('-u', '--nouserlist', action='store_true',
                                 dest='nouserlist', required=False,
                                 help='don\'t display userlist')
+    parser.add_argument('--no-rejoin-on-mention', action='store_true', dest='no_rejoin_on_mention',
+                                help='If set, mentions of the username will not cause the channel to be re-joined')
     parser.add_argument('-j', '--autojoin', action='store_true',
                                 dest='autojoin', required=False,
                                 help="Automatically join all remote channels")
@@ -885,6 +927,7 @@ def main() -> None:
 
     autojoin: bool = environ['AUTOJOIN'].lower() == 'true' if 'AUTOJOIN' in environ else args.autojoin
     nouserlist: bool = environ['NOUSERLIST'].lower() == 'true' if 'NOUSERLIST' in environ else args.nouserlist
+    no_rejoin_on_mention: bool = environ['NO_REJOIN_ON_MENTION'].lower() == 'true' if 'NO_REJOIN_ON_MENTION' in environ else args.no_rejoin_on_mention
 
     # Splitting ignored channels
     ignored_channels_str = environ.get('IGNORED_CHANNELS', args.ignored_channels)
@@ -966,6 +1009,7 @@ def main() -> None:
 
         clientsettings = ClientSettings(
             nouserlist=nouserlist,
+            no_rejoin_on_mention=no_rejoin_on_mention,
             autojoin=autojoin,
             provider=provider,
             ignored_channels=ignored_channels,
