@@ -416,12 +416,12 @@ class Slack:
         Obtain the history from the last known event and
         inject fake events as if the messages are coming now.
         '''
-        log('Fetching history...')
-
         if self._status.last_timestamp == 0:
             log('No last known timestamp. Unable to fetch history')
             return
 
+        log('Fetching history...')
+
         last_timestamp = self._status.last_timestamp
         FOUR_DAYS = 60 * 60 * 24 * 4
         if time() - last_timestamp > FOUR_DAYS:
@@ -488,7 +488,10 @@ class Slack:
                         ))
 
                 if response.has_more and response.response_metadata:
-                    cursor = response.response_metadata.next_cursor
+                    next_cursor = response.response_metadata.next_cursor
+                    if next_cursor == cursor:
+                        break
+                    cursor = next_cursor
                 else:
                     break
 
@@ -853,7 +856,6 @@ class Slack:
         try:
             events = await self.client.rtm_read()
         except Exception:
-            events = []
             log('Connecting to slack...')
             self.login_info = await self.client.rtm_connect(5)
             await self._history()
